@@ -1,4 +1,4 @@
-# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -15,12 +15,13 @@ from ebcli.core import io
 from ebcli.core.abstractcontroller import AbstractBaseController
 from ebcli.objects.exceptions import InvalidPlatformVersionError
 from ebcli.objects.platform import PlatformVersion
-from ebcli.operations import platformops, solution_stack_ops
+from ebcli.operations import platformops, platform_version_ops, solution_stack_ops
 from ebcli.resources.strings import strings, flag_text
 
 
 class PlatformShowController(AbstractBaseController):
     class Meta:
+        requires_directory_initialization = True
         label = 'platform show'
         aliases = ['show']
         aliases_only = True
@@ -50,9 +51,17 @@ class PlatformShowController(AbstractBaseController):
 
         if env_name:
             if want_solution_stack:
-                platform = platformops.get_environment_platform(app_name, env_name, want_solution_stack=want_solution_stack).name
+                platform = platformops.get_environment_platform(
+                    app_name,
+                    env_name,
+                    want_solution_stack=want_solution_stack
+                ).name
             else:
-                platform = platformops.get_environment_platform(app_name, env_name, want_solution_stack=want_solution_stack).arn
+                platform = platformops.get_environment_platform(
+                    app_name,
+                    env_name,
+                    want_solution_stack=want_solution_stack
+                ).arn
 
             io.echo()
             io.echo('Platform info for environment "{env_name}":'
@@ -65,21 +74,34 @@ class PlatformShowController(AbstractBaseController):
 
     def get_latest_solution_stack(self, solution_stack):
         full_platform = solution_stack_ops.find_solution_stack_from_string(solution_stack)
-        latest_platform = solution_stack_ops.find_solution_stack_from_string(full_platform.platform_shorthand, find_newer=True)
+        latest_platform = solution_stack_ops.find_solution_stack_from_string(
+            full_platform.platform_shorthand,
+            find_newer=True
+        )
 
         return full_platform.name, latest_platform.name
 
     def get_latest_custom_platform(self, platform_arn):
-        latest_platform = platformops.get_latest_custom_platform(platform_arn)
+        latest_platform = platform_version_ops.get_latest_custom_platform_version(platform_arn)
 
         return platform_arn, latest_platform.platform_shorthand
 
 
 class GenericPlatformStatusController(AbstractBaseController):
     class Meta:
+        is_platform_workspace_only_command = True
+        requires_directory_initialization = True
         description = strings['platformshowversion.info']
         arguments = [
-            (['version'], dict(action='store', nargs='?', default=None, help=flag_text['platformshowversion.version'])),
+            (
+                ['version'],
+                dict(
+                    action='store',
+                    nargs='?',
+                    default=None,
+                    help=flag_text['platformshowversion.version']
+                )
+            ),
         ]
         epilog = strings['platformshowversion.epilog']
 
