@@ -1,4 +1,4 @@
-# Copyright 2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
+# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
 #
 # Licensed under the Apache License, Version 2.0 (the "License"). You
 # may not use this file except in compliance with the License. A copy of
@@ -21,7 +21,6 @@ from ebcli.resources.statics import namespaces, option_names
 from ebcli.objects.solutionstack import SolutionStack
 from ebcli.objects.exceptions import NotFoundError, InvalidStateError, \
     AlreadyExistsError
-from ebcli.objects.tier import Tier
 from ebcli.lib import aws
 from ebcli.lib.aws import InvalidParameterValueError
 from ebcli.objects.event import Event
@@ -47,6 +46,28 @@ def delete_platform(arn):
                           DeleteResources=True)
 
 
+def list_platform_branches(filters=None):
+    LOG.debug('Inside list_platform_branches api wrapper')
+    kwargs = dict()
+
+    if filters:
+        kwargs['Filters'] = filters
+
+    next_token = None
+    platform_branches = []
+
+    while True:
+        next_platform_branches, next_token = _list_platform_branches(**kwargs)
+        platform_branches += next_platform_branches
+
+        if next_token is None:
+            break
+        else:
+            kwargs['NextToken'] = next_token
+
+    return platform_branches
+
+
 def list_platform_versions(filters=None):
     kwargs = dict()
 
@@ -70,6 +91,18 @@ def describe_platform_version(arn):
                           PlatformArn=arn)['PlatformDescription']
 
 
+def _list_platform_branches(**kwargs):
+    response = _make_api_call(
+        'list_platform_branches',
+        **kwargs
+    )
+
+    platform_branches = response.get('PlatformBranchSummaryList', [])
+    next_token = response.get('NextToken')
+
+    return platform_branches, next_token
+
+
 def _list_platform_versions(kwargs, nextToken=None):
     if nextToken is not None:
         # Sleep for 100ms before pulling the next page
@@ -90,11 +123,12 @@ def _list_platform_versions(kwargs, nextToken=None):
     return platforms, nextToken
 
 
-def create_application(app_name, descrip):
+def create_application(app_name, descrip, tags=[]):
     LOG.debug('Inside create_application api wrapper')
     try:
         result = _make_api_call('create_application',
                                 ApplicationName=app_name,
+                                Tags=tags,
                                 Description=descrip)
     except InvalidParameterValueError as e:
         string = responses['app.exists'].replace('{app-name}', app_name)
@@ -106,12 +140,25 @@ def create_application(app_name, descrip):
     return result
 
 
-def create_platform_version(platform_name, version, s3_bucket, s3_key, instance_profile, key_name, instance_type, vpc = None):
+def create_platform_version(
+        platform_name,
+        version,
+        s3_bucket,
+        s3_key,
+        instance_profile,
+        key_name,
+        instance_type,
+        tags=[],
+        vpc=None
+):
     kwargs = dict()
 
     if s3_bucket and s3_key:
         kwargs['PlatformDefinitionBundle'] = {'S3Bucket': s3_bucket, 'S3Key': s3_key}
 
+    if tags is not None:
+        kwargs['Tags'] = tags
+
     option_settings = []
 
     if instance_profile:
@@ -152,14 +199,12 @@ def create_platform_version(platform_name, version, s3_bucket, s3_key, instance_
                 'Value': 'true'
             })
 
-    # Always enable healthd for the Platform Builder environment
     option_settings.append({
         'Namespace': namespaces.HEALTH_SYSTEM,
         'OptionName': option_names.SYSTEM_TYPE,
         'Value': 'enhanced'
     })
 
-    # Attach service role
     option_settings.append({
         'Namespace': namespaces.ENVIRONMENT,
         'OptionName': option_names.SERVICE_ROLE,
@@ -174,8 +219,17 @@ def create_platform_version(platform_name, version, s3_bucket, s3_key, instance_
                           **kwargs)
 
 
-def create_application_version(app_name, vers_label, descrip, s3_bucket,
-                               s3_key, process=False, repository=None, commit_id=None, build_configuration=None):
+def create_application_version(
+        app_name,
+        vers_label,
+        descrip,
+        s3_bucket,
+        s3_key,
+        process=False,
+        repository=None,
+        commit_id=None,
+        build_configuration=None
+):
     kwargs = dict()
     kwargs['Process'] = process
     if descrip is not None:
@@ -189,9 +243,11 @@ def create_application_version(app_name, vers_label, descrip, s3_bucket,
                                                 'SourceRepository': 'S3',
                                                 'SourceLocation': "{0}/{1}".format(s3_bucket, s3_key)}
     elif repository and commit_id:
-        kwargs['SourceBuildInformation'] = {'SourceType': 'Git',
-                                  'SourceRepository': 'CodeCommit',
-                                  'SourceLocation': "{0}/{1}".format(repository, commit_id)}
+        kwargs['SourceBuildInformation'] = {
+            'SourceType': 'Git',
+            'SourceRepository': 'CodeCommit',
+            'SourceLocation': "{0}/{1}".format(repository, commit_id)
+        }
         kwargs['Process'] = True
 
     if build_configuration is not None:
@@ -217,10 +273,8 @@ def create_environment(environment):
     kwargs = environment.convert_to_kwargs()
 
     if environment.database:
-        # need to know region for database string
         region = aws.get_region_name()
 
-        # Database is a dictionary
         kwargs['TemplateSpecification'] = {
             'TemplateSnippets': [
                 {'SnippetName': 'RdsExtensionEB',
@@ -233,8 +287,7 @@ def create_environment(environment):
 
     result = _make_api_call('create_environment', **kwargs)
 
-    # convert to object
-    env = _api_to_environment(result)
+    env = Environment.json_to_environment_object(result)
     request_id = result['ResponseMetadata']['RequestId']
     return env, request_id
 
@@ -249,46 +302,9 @@ def clone_environment(clone):
 
     result = _make_api_call('create_environment', **kwargs)
 
-    # convert to object
-    env = _api_to_environment(result)
+    environment = Environment.json_to_environment_object(result)
     request_id = result['ResponseMetadata']['RequestId']
-    return env, request_id
-
-
-def _api_to_environment(api_dict, want_solution_stack = False):
-    # Convert solution_stack and tier to objects
-    try:
-        if want_solution_stack:
-            solution_stack_name = api_dict['SolutionStackName']
-            platform = SolutionStack(solution_stack_name)
-        else:
-            platform_arn = api_dict['PlatformArn']
-            platform = PlatformVersion(platform_arn)
-    except KeyError:
-        platform = SolutionStack(api_dict['SolutionStackName'])
-
-    tier = api_dict['Tier']
-    tier = Tier(tier['Name'], tier['Type'], tier['Version'])
-
-    env = Environment(
-        version_label=api_dict.get('VersionLabel'),
-        status=api_dict.get('Status'),
-        app_name=api_dict.get('ApplicationName'),
-        health=api_dict.get('Health'),
-        id=api_dict.get('EnvironmentId'),
-        date_updated=api_dict.get('DateUpdated'),
-        platform=platform,
-        description=api_dict.get('Description'),
-        name=api_dict.get('EnvironmentName'),
-        date_created=api_dict.get('DateCreated'),
-        tier=tier,
-        cname=api_dict.get('CNAME', 'UNKNOWN'),
-        option_settings=api_dict.get('OptionSettings'),
-        is_abortable=api_dict.get('AbortableOperationInProgress', False),
-        environment_links=api_dict.get('EnvironmentLinks'),
-        environment_arn=api_dict.get('EnvironmentArn')
-    )
-    return env
+    return environment, request_id
 
 
 def delete_application(app_name):
@@ -362,6 +378,12 @@ def describe_configuration_settings(app_name, env_name):
     return result['ConfigurationSettings'][0]
 
 
+def get_application_names():
+    applications = get_all_applications()
+
+    return [application.name for application in applications]
+
+
 def get_option_setting_from_environment(app_name, env_name, namespace, option):
     env = describe_configuration_settings(app_name, env_name)
     try:
@@ -426,7 +448,8 @@ def get_application_versions(app_name, version_labels=None, max_records=None, ne
     if max_records:
         kwargs['MaxRecords'] = max_records
     if next_token:
-        time.sleep(0.1) # To avoid throttling we sleep for 100ms before requesting the next page
+        # To avoid throttling we sleep for 100ms before requesting the next page
+        time.sleep(0.1)
         kwargs['NextToken'] = next_token
     result = _make_api_call('describe_application_versions',
                             ApplicationName=app_name,
@@ -434,6 +457,13 @@ def get_application_versions(app_name, version_labels=None, max_records=None, ne
     return result
 
 
+def application_version_exists(app_name, version_label):
+    app_versions = get_application_versions(app_name, version_labels=[version_label])['ApplicationVersions']
+
+    if len(app_versions) > 0:
+        return app_versions[0]
+
+
 def get_all_applications():
     LOG.debug('Inside get_all_applications api wrapper')
     result = _make_api_call('describe_applications')
@@ -487,24 +517,32 @@ def get_app_environments(app_name, include_deleted=False, deleted_back_to=None):
                             ApplicationName=app_name,
                             IncludeDeleted=include_deleted,
                             **kwargs)
-    # convert to objects
-    envs = [_api_to_environment(env) for env in result['Environments']]
 
-    return envs
+    return Environment.json_to_environment_objects_array(result['Environments'])
+
+
+def get_all_environment_names():
+    environments = get_all_environments()
+
+    return [environment.name for environment in environments]
 
 
 def get_all_environments():
     LOG.debug('Inside get_all_environments api wrapper')
     result = _make_api_call('describe_environments',
                             IncludeDeleted=False)
-    # convert to object
-    envs = []
-    for env in result['Environments']:
-        envs.append(_api_to_environment(env))
-    return envs
+
+    return Environment.json_to_environment_objects_array(result['Environments'])
 
 
-def get_environment(app_name=None, env_name=None, env_id=None, include_deleted=False, deleted_back_to=None, want_solution_stack=False):
+def get_environment(
+        app_name=None,
+        env_name=None,
+        env_id=None,
+        include_deleted=False,
+        deleted_back_to=None,
+        want_solution_stack=False
+):
     LOG.debug('Inside get_environment api wrapper')
 
     kwargs = {}
@@ -526,20 +564,34 @@ def get_environment(app_name=None, env_name=None, env_id=None, include_deleted=F
         env_str = env_id if env_name is None else env_name
         raise NotFoundError('Environment "' + env_str + '" not Found.')
     else:
-        return _api_to_environment(envs[0], want_solution_stack)
+        return Environment.json_to_environment_object(envs[0], want_solution_stack)
 
 
-def get_environments(env_names=[]):
+def get_environment_names(app_name):
+    environments = get_app_environments(app_name)
+    return [environment.name for environment in environments]
+
+
+def get_app_version_labels(app_name):
+    app_versions = get_application_versions(app_name)['ApplicationVersions']
+
+    return [app_version['VersionLabel'] for app_version in app_versions]
+
+
+def get_environments(env_names=None):
     LOG.debug('Inside get_environments api wrapper')
     result = _make_api_call('describe_environments',
-                            EnvironmentNames=env_names,
+                            EnvironmentNames=env_names or [],
                             IncludeDeleted=False)
 
-    envs = result['Environments']
-    if len(envs) < 1:
-        raise NotFoundError('Could not find any environments '
-                            'from the list: [' + env_names + ']')
-    return [_api_to_environment(env) for env in envs]
+    environments = result['Environments']
+    if not environments and env_names:
+        raise NotFoundError(
+            'Could not find any environments from the list: {}'.format(
+                ', '.join(env_names)
+            )
+        )
+    return Environment.json_to_environment_objects_array(environments)
 
 
 def get_environment_settings(app_name, env_name):
@@ -548,7 +600,7 @@ def get_environment_settings(app_name, env_name):
                             ApplicationName=app_name,
                             EnvironmentName=env_name)
 
-    return _api_to_environment(result['ConfigurationSettings'][0])
+    return Environment.json_to_environment_object(result['ConfigurationSettings'][0])
 
 
 def get_environment_resources(env_name):
@@ -561,10 +613,8 @@ def get_environment_resources(env_name):
 def get_new_events(app_name, env_name, request_id,
                    last_event_time=None, version_label=None, platform_arn=None):
     LOG.debug('Inside get_new_events api wrapper')
-    # make call
+
     if last_event_time is not None:
-        # In python 2 time is a datetime, in 3 it is a string
-        ## Convert to string for compatibility
         time = last_event_time
         new_time = time + datetime.timedelta(0, 0, 1000)
     else:
@@ -586,23 +636,7 @@ def get_new_events(app_name, env_name, request_id,
     result = _make_api_call('describe_events',
                             **kwargs)
 
-    # convert to object
-    events = []
-    for event in result['Events']:
-        events.append(
-            Event(
-                app_name=event.get('ApplicationName'),
-                environment_name=event.get('EnvironmentName'),
-                event_date=event.get('EventDate'),
-                message=event.get('Message'),
-                platform=platform_arn,
-                request_id=event.get('RequestId'),
-                severity=event.get('Severity'),
-                version_label=event.get('VersionLabel')
-            )
-        )
-
-    return events
+    return Event.json_to_event_objects(result['Events'])
 
 
 def get_storage_location():
@@ -704,7 +738,7 @@ def terminate_environment(env_name, force_terminate=False):
 
 
 def create_configuration_template(app_name, env_name, template_name,
-                                  description):
+                                  description, tags):
     kwargs = {
         'TemplateName': template_name,
         'ApplicationName': app_name,
@@ -712,8 +746,8 @@ def create_configuration_template(app_name, env_name, template_name,
         'TemplateSpecification':
             {'TemplateSource':
                 {'EnvironmentName': env_name}},
+        'Tags': tags
     }
-
     try:
         result = _make_api_call('create_configuration_template', **kwargs)
     except InvalidParameterValueError as e:
@@ -736,15 +770,17 @@ def validate_template(app_name, template_name, platform=None):
     kwargs = {}
     if platform:
         if PlatformVersion.is_valid_arn(platform):
-            kwargs['TemplateSpecification'] = \
-                {'TemplateSource':
-                     {'PlatformArn': platform}}
+            kwargs['TemplateSpecification'] = {
+                'TemplateSource': {
+                    'PlatformArn': platform
+                }
+            }
         else:
-            kwargs['TemplateSpecification'] = \
-                {'TemplateSource':
-                    {'SolutionStackName': platform}}
-
-
+            kwargs['TemplateSpecification'] = {
+                'TemplateSource': {
+                    'SolutionStackName': platform
+                }
+            }
 
     result = _make_api_call('validate_configuration_settings',
                             ApplicationName=app_name,
@@ -778,6 +814,10 @@ def get_environment_health(env_name, attributes=None):
     return result
 
 
+def get_environment_tier_definition():
+    return get_environment(app_name=None, env_name=None).tier
+
+
 def get_instance_health(env_name, next_token=None, attributes=None):
     if attributes is None:
         attributes = [
@@ -834,19 +874,19 @@ def get_environment_arn(env_name):
     return get_environments([env_name])[0].environment_arn
 
 
-def list_tags_for_resource(env_name):
+def list_tags_for_resource(resource_arn):
     response = _make_api_call(
         'list_tags_for_resource',
-        ResourceArn=get_environment_arn(env_name)
+        ResourceArn=resource_arn
     )
 
     return sorted(response['ResourceTags'], key=lambda tag: tag['Key'])
 
 
-def update_tags_for_resource(env_name, tags_to_add, tags_to_remove):
+def update_tags_for_resource(resource_arn, tags_to_add, tags_to_remove):
     response = _make_api_call(
         'update_tags_for_resource',
-        ResourceArn=get_environment_arn(env_name),
+        ResourceArn=resource_arn,
         TagsToAdd=tags_to_add,
         TagsToRemove=tags_to_remove
     )
